/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file DataTransferCommand.h
 *
 * @date 2012-03-24
 * @author Philipp Kegel
 */

#ifndef DATATRANSFERCOMMAND_H_
#define DATATRANSFERCOMMAND_H_

#include "Command.h"

#include <dcl/DataTransfer.h>
#include <dcl/DCLTypes.h>
#include <dcl/Host.h>

#define __CL_ENABLE_EXCEPTIONS
#include <CL/cl.hpp>

#include <memory>

namespace dcld {

namespace command {

class DataTransferCommand: public Command {
public:
    DataTransferCommand(
            dcl::Host&              host,
            dcl::object_id          commandId,
            const cl::UserEvent&    event);

    void execute(
            cl_int errcode);

protected:
    virtual std::shared_ptr<dcl::DataTransfer> startDataTransfer() = 0;
};

} /* namespace command */

} /* namespace dcld */

#endif /* DATATRANSFERCOMMAND_H_ */
