/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file ReadWriteCommand.cpp
 *
 * @date 2012-03-24
 * @author Philipp Kegel
 */

#include "ReadWriteCommand.h"

#include "DataTransferCommand.h"

#include <dcl/DataTransfer.h>
#include <dcl/DCLTypes.h>
#include <dcl/Host.h>

#define __CL_ENABLE_EXCEPTIONS
#include <CL/cl.hpp>

#include <cstddef>
#include <memory>

namespace dcld {

namespace command {

ReadBufferCommand::ReadBufferCommand(
        dcl::Host& host,
        dcl::object_id commandId,
        size_t cb,
        const void *ptr,
        const cl::UserEvent& event) :
    DataTransferCommand(host, commandId, event), _cb(cb), _ptr(ptr) {
}

std::shared_ptr<dcl::DataTransfer> ReadBufferCommand::startDataTransfer() {
    /* Start data sending */
    return _host.sendData(_cb, _ptr);
}

/******************************************************************************/

WriteBufferCommand::WriteBufferCommand(
        dcl::Host& host,
        dcl::object_id commandId,
        size_t cb,
        void *ptr,
        const cl::UserEvent& event) :
    DataTransferCommand(host, commandId, event), _cb(cb), _ptr(ptr) {
}

std::shared_ptr<dcl::DataTransfer> WriteBufferCommand::startDataTransfer() {
    /* Start data receipt */
    return _host.receiveData(_cb, _ptr);
}

} /* namespace command */

} /* namespace dcld */
