/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/*!
 * \file CreateEvent.h
 *
 * \date 2011-07-11
 * \author Philipp Kegel
 */

#ifndef CREATEEVENT_H_
#define CREATEEVENT_H_

#include "Request.h"

#include <dcl/DCLTypes.h>

#include <vector>

namespace dclgcf {

namespace message {

/*!
 * \brief A request message for creating an event.
 *
 * \date    2012-07-17
 * \author  Philipp Kegel
 */
class CreateEvent: public Request {
public:
	CreateEvent(
			dcl::object_id                     contextId,
			dcl::object_id                     eventId,
			const std::vector<dcl::object_id>& memObjectIds);
	CreateEvent(
	        const CreateEvent& rhs);
	virtual ~CreateEvent();

	dcl::object_id contextId() const;
	dcl::object_id eventId() const;
	const std::vector<dcl::object_id>& memObjectIds() const;

	DECLARE_SERIALIZABLE_PUBLIC(CreateEvent, 100 + CREATE_EVENT)

private:
	dcl::object_id _contextId;
	dcl::object_id _eventId;
	std::vector<dcl::object_id> _memObjectIds;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(CreateEvent)
};

} /* namespace message */

} /* namespace dclgcf */

#endif /* CREATEEVENT_H_ */
