/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file EnqueueMapBuffer.h
 *
 * @date 2012-08-08
 * @author Philipp Kegel
 */

#ifndef ENQUEUEMAPBUFFER_H_
#define ENQUEUEMAPBUFFER_H_

#include "Request.h"

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <cstddef>
#include <vector>

namespace dclgcf {

namespace message {

class EnqueueMapBuffer : public Request{
public:
    EnqueueMapBuffer(
            dcl::object_id                      commandQueueId,
            dcl::object_id                      commandId,
            dcl::object_id                      bufferId,
            bool                                blocking_map,
            cl_map_flags                        map_flags,
            size_t                              offset,
            size_t                              cb,
            const std::vector<dcl::object_id> * eventIdWaitList = nullptr,
            bool                                event = false);
    EnqueueMapBuffer(
            const EnqueueMapBuffer& rhs);

    dcl::object_id commandQueueId() const;
    dcl::object_id commandId() const;
    dcl::object_id bufferId() const;
    bool blocking() const;
    cl_map_flags mapFlags() const;
    size_t offset() const;
    size_t cb() const;
    const std::vector<dcl::object_id>& eventIdWaitList() const;
    bool event() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueMapBuffer, 100 + ENQUEUE_MAP_BUFFER)

private:
    dcl::object_id _commandQueueId;
    dcl::object_id _commandId;
    dcl::object_id _bufferId;
    gcf_bool _blocking;
    gcf_uint64 _mapFlags;
    gcf_uint32 _offset;
    gcf_uint32 _cb;
    std::vector<dcl::object_id> _eventIdWaitList;
    gcf_bool _event;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueMapBuffer)
};

} /* namespace message */

} /* namespace dclgcf */

#endif /* ENQUEUEMAPBUFFER_H_ */
