/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file EnqueueWaitForEvents.h
 *
 * @date 2011-12-18
 * @author Philipp Kegel
 */

#ifndef ENQUEUEWAITFOREVENTS_H_
#define ENQUEUEWAITFOREVENTS_H_

#include "Request.h"

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>

#include <vector>

namespace dclgcf {

namespace message {

class EnqueueWaitForEvents: public Request {
public:
    EnqueueWaitForEvents(
            dcl::object_id                      commandQueueId,
            const std::vector<dcl::object_id>&  eventIdList);
    EnqueueWaitForEvents(
            const EnqueueWaitForEvents& rhs);

    dcl::object_id commandQueueId() const;
    const std::vector<dcl::object_id>& eventIdList() const;

    DECLARE_SERIALIZABLE_PUBLIC(EnqueueWaitForEvents, 100 + ENQUEUE_WAIT_FOR_EVENTS)

private:
    dcl::object_id _commandQueueId;
    std::vector<dcl::object_id> _eventIdList;

    DECLARE_SERIALIZABLE_EVENT_PRIVATE(EnqueueWaitForEvents)
};

} /* namespace message */

} /* namespace dclgcf */

#endif /* ENQUEUEWAITFOREVENTS_H_ */
