/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file ContextErrorMessage.cpp
 *
 * @date 2012-03-27
 * @author Sebastian Pribnow
 * @author Philipp Kegel
 */

#include "ContextErrorMessage.h"

#include <dcl/Binary.h>
#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <cassert>
#include <cstddef>
#include <string>

namespace gcf {

template<>
size_t writeCustomSTR<dclgcf::message::ContextErrorMessage> (
		dclgcf::message::ContextErrorMessage *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	dcl::Binary& argValue =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::ContextErrorMessage, dcl::Binary,
					gcf::ST_Custom1>(object, gstr);

	buf << argValue;

	return buf.getLength();
}

template<>
size_t readCustomSTR<dclgcf::message::ContextErrorMessage> (
		dclgcf::message::ContextErrorMessage *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		const char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	dcl::Binary& argValue =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::ContextErrorMessage, dcl::Binary,
					gcf::ST_Custom1>(object, gstr);

	buf >> argValue;

	return buf.getLength();
}

} /* namespace gcf */

/******************************************************************************/

namespace dclgcf {

namespace message {

ContextErrorMessage::ContextErrorMessage() {
	initSerializable();
}

ContextErrorMessage::ContextErrorMessage(dcl::object_id contextId,
		const std::string& errInfo, const dcl::Binary& privateInfo) :
	_contextId(contextId), _errorInfo(errInfo), _privateInfo(privateInfo)
{
	initSerializable();
}

ContextErrorMessage::ContextErrorMessage(const ContextErrorMessage& rhs) :
	_contextId(rhs._contextId), _errorInfo(rhs._errorInfo),
	_privateInfo(rhs._privateInfo)
{
	initSerializable();
}

ContextErrorMessage::~ContextErrorMessage() { }

dcl::object_id ContextErrorMessage::contextId() const {
	return _contextId;
}

const std::string& ContextErrorMessage::errorInfo() const {
	return _errorInfo;
}

const dcl::Binary& ContextErrorMessage::privateInfo() const {
	return _privateInfo;
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_EVENT(
		dclgcf::message::ContextErrorMessage,
		ADD_ATTRIBUTE_DEFAULT(dclgcf::message::ContextErrorMessage, _contextId);
		ADD_ATTRIBUTE_DEFAULT(dclgcf::message::ContextErrorMessage, _errorInfo);
		ADD_ATTRIBUTE_CUSTOM(dclgcf::message::ContextErrorMessage, _privateInfo,
						gcf::ST_Custom1, dcl::Binary, 256, gcf::Reliable,
						gcf::Public);
		)
