/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file    CreateKernelsInProgram.cpp
 *
 * @date    20.06.2011
 * @author  Karsten Jeschkies
 */

#include <dclgcf/message/CreateKernelsInProgram.h>
#include <dclgcf/message/Request.h>

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <gcf/serialization/ByteBuffer.h>

#include <cassert>
#include <cstddef>
#include <vector>

namespace gcf {

template<>
size_t writeCustomSTR<dclgcf::message::CreateKernelsInProgram> (
		dclgcf::message::CreateKernelsInProgram *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	std::vector<dcl::object_id>& kernelIds =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::CreateKernelsInProgram, std::vector<dcl::object_id>,
					gcf::ST_Custom1>(object, gstr);

	buf << kernelIds;

	return buf.getLength();
}

template<>
size_t readCustomSTR<dclgcf::message::CreateKernelsInProgram> (
		dclgcf::message::CreateKernelsInProgram *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		const char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	std::vector<dcl::object_id>& kernelIds =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::CreateKernelsInProgram, std::vector<dcl::object_id>,
					gcf::ST_Custom1>(object, gstr);

	kernelIds.clear();
	buf >> kernelIds;

	return buf.getLength();
}

} /* namespace gcf */

/******************************************************************************/

namespace dclgcf {

namespace message {

CreateKernelsInProgram::CreateKernelsInProgram(
		dcl::object_id programId,
		const std::vector<dcl::object_id>& kernelIds) :
	_programId(programId), _kernelIds(kernelIds) {
	initSerializable();
}

CreateKernelsInProgram::CreateKernelsInProgram(
		const CreateKernelsInProgram& rhs) :
	Request(rhs), _programId(rhs._programId), _kernelIds(rhs._kernelIds) {
	initSerializable();
}

CreateKernelsInProgram::CreateKernelsInProgram() :
	_programId(0) {
	initSerializable();
}

const std::vector<dcl::object_id>& CreateKernelsInProgram::kernelIds() const {
	return _kernelIds;
}

dcl::object_id CreateKernelsInProgram::programId() const {
	return _programId;
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
        dclgcf::message::CreateKernelsInProgram,
        dclgcf::message::Request,
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::CreateKernelsInProgram, _programId);
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::CreateKernelsInProgram, _kernelIds,
                gcf::ST_Custom1, std::vector<dcl::object_id>, 256,
                gcf::Reliable, gcf::Public);
        )
