/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/*!
 * \file    DeviceInfosResponse.cpp
 *
 * \date    2011-05-19
 * \author  Marius Holzwarth
 * \author  Philipp Kegel
 */

#include "DeviceInfosResponse.h"

#include <dclgcf/message/Request.h>
#include <dclgcf/message/Response.h>

#include <dcl/Binary.h>

#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <gcf/serialization/ByteBuffer.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <cassert>
#include <cstddef>
#include <map>
#include <utility>

namespace gcf {

template<>
size_t writeCustomSTR<dclgcf::message::DeviceInfosResponse> (
		dclgcf::message::DeviceInfosResponse *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	std::map<cl_device_info, dcl::Binary>& params =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::DeviceInfosResponse, std::map<cl_device_info, dcl::Binary>,
					gcf::ST_Custom1>(object, gstr);

	/* Serialize device information set */
	buf << params.size();
	for (auto i : params) {
	    buf << i.first << i.second;
	}

	return buf.getLength();
}

template<>
size_t readCustomSTR<dclgcf::message::DeviceInfosResponse> (
		dclgcf::message::DeviceInfosResponse *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		const char *buffer) {
    std::map<cl_device_info, dcl::Binary>::size_type size = 0;

	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	std::map<cl_device_info, dcl::Binary>& params =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::DeviceInfosResponse, std::map<cl_device_info, dcl::Binary>,
					gcf::ST_Custom1>(object, gstr);

    /* Deserialize device information set */
	params.clear();
	buf >> size;
    for (std::map<cl_device_info, dcl::Binary>::size_type i = 0; i != size; ++i) {
        cl_device_info paramName;
        dcl::Binary param;

        buf >> paramName >> param;

        params.insert(std::make_pair(paramName, param));
    }

	return buf.getLength();
}

} /* namespace gcf */

/******************************************************************************/

namespace dclgcf {

namespace message {

DeviceInfosResponse::DeviceInfosResponse() {
    initSerializable();
}

DeviceInfosResponse::DeviceInfosResponse(
        const Request& request,
        const std::map<cl_device_info, const dcl::Binary>& params) :
    Response(request), _params(params) {
    initSerializable();
}

DeviceInfosResponse::DeviceInfosResponse(
        const Request& request,
        cl_device_info paramName,
        size_t size,
        const void *value) :
    Response(request) {
    _params.insert(std::make_pair(paramName, dcl::Binary(size, value)));
    initSerializable();
}

DeviceInfosResponse::DeviceInfosResponse(
        const DeviceInfosResponse& rhs) :
    Response(rhs), _params(rhs._params) {
    initSerializable();
}

DeviceInfosResponse::~DeviceInfosResponse() { }

dcl::Binary DeviceInfosResponse::param(
        cl_device_info paramName) const {
    return _params.at(paramName);
}

std::map<cl_device_info, const dcl::Binary> DeviceInfosResponse::params() const {
    return _params;
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
        dclgcf::message::DeviceInfosResponse,
        dclgcf::message::Response,
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::DeviceInfosResponse, _params,
                gcf::ST_Custom1, dcl::Binary, 1024, gcf::Reliable, gcf::Public);
        )
