/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/*!
 * \file EventProfilingInfosResponse.cpp
 *
 * \date 2013-10-27
 * \author Philipp Kegel
 */

#include <dclgcf/message/EventProfilingInfosResponse.h>

#include <dclgcf/message/Request.h>
#include <dclgcf/message/Response.h>

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <cassert>

namespace gcf {

template<>
size_t writeCustomSTR<dclgcf::message::EventProfilingInfosReponse> (
		dclgcf::message::EventProfilingInfosReponse *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	gcf_uint64& profilingInfo =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::EventProfilingInfosReponse, gcf_uint64,
					gcf::ST_Custom1>(object, gstr);

	buf << profilingInfo;

	return buf.getLength();
}

template<>
size_t readCustomSTR<dclgcf::message::EventProfilingInfosReponse> (
		dclgcf::message::EventProfilingInfosReponse *object,
		const gcf::GenericSerializeTemplateRecord& gstr,
		const char *buffer) {
	assert(gstr.getType() == gcf::ST_Custom1);

	gcf::ByteBuffer buf(buffer, gstr.getLength());
	gcf_uint64& profilingInfo =
			*gcf::GenericSerializeTemplateRecord::getAttribute<
					dclgcf::message::EventProfilingInfosReponse, gcf_uint64,
					gcf::ST_Custom1>(object, gstr);

	buf >> profilingInfo;

	return buf.getLength();
}

} /* namespace gcf */

/******************************************************************************/

namespace dclgcf {

namespace message {

EventProfilingInfosReponse::EventProfilingInfosReponse() {
	initSerializable();
}

EventProfilingInfosReponse::EventProfilingInfosReponse(const Request& request,
        cl_ulong received, cl_ulong queued, cl_ulong submit, cl_ulong start,
        cl_ulong end) :
	Response(request),
    _received(static_cast<gcf_uint64>(received)),
	_queued(static_cast<gcf_uint64>(queued)),
	_submit(static_cast<gcf_uint64>(submit)),
	_start(static_cast<gcf_uint64>(start)),
	_end(static_cast<gcf_uint64>(end))
{
	initSerializable();
}

EventProfilingInfosReponse::EventProfilingInfosReponse(const EventProfilingInfosReponse& rhs) :
	Response(rhs), _received(rhs._received), _queued(rhs._queued),
	_submit(rhs._submit), _start(rhs._start), _end(rhs._end)
{
	initSerializable();
}

EventProfilingInfosReponse::~EventProfilingInfosReponse() { }

cl_ulong EventProfilingInfosReponse::received() const {
    return static_cast<cl_ulong>(_received);
}

cl_ulong EventProfilingInfosReponse::queued() const {
	return static_cast<cl_ulong>(_queued);
}

cl_ulong EventProfilingInfosReponse::submit() const {
	return static_cast<cl_ulong>(_submit);
}

cl_ulong EventProfilingInfosReponse::start() const {
	return static_cast<cl_ulong>(_start);
}

cl_ulong EventProfilingInfosReponse::end() const {
	return static_cast<cl_ulong>(_end);
}

} /* namespace message */

} /* namespace dclgcf */

/*
 * GCF does not provide automatic serialization for gcf_uint64.
 * Hence, values of this type have to be serialized explicitly.
 */
IMPLEMENT_SERIALIZABLE_DERIVED_EVENT(
        dclgcf::message::EventProfilingInfosReponse,
        dclgcf::message::Response,
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::EventProfilingInfosReponse, _received,
                gcf::ST_Custom1, gcf_uint64, 8, gcf::Reliable, gcf::Public);
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::EventProfilingInfosReponse, _queued,
                gcf::ST_Custom1, gcf_uint64, 8, gcf::Reliable, gcf::Public);
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::EventProfilingInfosReponse, _submit,
                gcf::ST_Custom1, gcf_uint64, 8, gcf::Reliable, gcf::Public);
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::EventProfilingInfosReponse, _start,
                gcf::ST_Custom1, gcf_uint64, 8, gcf::Reliable, gcf::Public);
        ADD_ATTRIBUTE_CUSTOM(dclgcf::message::EventProfilingInfosReponse, _end,
                gcf::ST_Custom1, gcf_uint64, 8, gcf::Reliable, gcf::Public);
        )
