/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @brief ProcessMessage.h
 *
 * @date 2011-07-15
 * @author Philipp Kegel
 */

#ifndef PROCESSMESSAGE_H_
#define PROCESSMESSAGE_H_

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

namespace dclgcf {

namespace message {

/* TODO Define unified process identification message
 * Replace HostIdentificationMessage and ComputeNodeIdentificationMessage by
 * ProcessIdentificationMessage containing process type */

/**
 * This message is send by a process to identify itself as a dOpenCL host.
 */
class HostIdentificationMessage: public gcf::Serializable {
public:
	HostIdentificationMessage(
	        const HostIdentificationMessage& rhs);
	virtual ~HostIdentificationMessage();

	DECLARE_SERIALIZABLE_PUBLIC(HostIdentificationMessage, 501)

private:
	DECLARE_SERIALIZABLE_EVENT_PRIVATE(HostIdentificationMessage)
};

/******************************************************************************/

/**
 * This message is send by a process to identify itself as a dOpenCL compute node.
 */
class ComputeNodeIdentificationMessage: public gcf::Serializable {
public:
	ComputeNodeIdentificationMessage(
	        const ComputeNodeIdentificationMessage& rhs);
	virtual ~ComputeNodeIdentificationMessage();

	DECLARE_SERIALIZABLE_PUBLIC(ComputeNodeIdentificationMessage, 502)

private:
	DECLARE_SERIALIZABLE_EVENT_PRIVATE(ComputeNodeIdentificationMessage)
};

/******************************************************************************/

/**
 * This message is send by a process when an incoming connection has been
 * approved
 */
class ConnectionApprovedMessage: public gcf::Serializable {
public:
	ConnectionApprovedMessage(
	        const ConnectionApprovedMessage& rhs);
	virtual ~ConnectionApprovedMessage();

	DECLARE_SERIALIZABLE_PUBLIC(ConnectionApprovedMessage, 510)

private:
	DECLARE_SERIALIZABLE_EVENT_PRIVATE(ConnectionApprovedMessage)
};

/******************************************************************************/

/* TODO Define ConnectionRejectedMessage */

} /* namespace message */

} /* namespace dclgcf */

#endif /* PROCESSMESSAGE_H_ */
