/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file ProgramBuildMessage.cpp
 *
 * Messages regarding program build.
 *
 * @date 2011-12-23
 * @author Philipp Kegel
 */

#include "ProgramBuildMessage.h"

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/GenericSerializerImplNew.cpp>
#include <gcf/Serializable.h>

#include <iterator>
#include <vector>

namespace dclgcf {

namespace message {

ProgramBuildMessage::ProgramBuildMessage() {
	initSerializable();
}

ProgramBuildMessage::ProgramBuildMessage(
		dcl::object_id programId,
		const std::vector<dcl::object_id>& deviceIds,
		const std::vector<cl_build_status>& buildStatus):
	_programBuildId(programId), _deviceIds(deviceIds) {
    std::vector<cl_build_status>::const_iterator src;
    std::vector<gcf_uint32>::iterator dst;

	initSerializable();

    _buildStatus.resize(buildStatus.size());
    src = std::begin(buildStatus);
    dst = std::begin(_buildStatus);

    while (src != std::end(buildStatus)) {
        *dst++ = static_cast<gcf_uint32>(*src++);
    }
}

ProgramBuildMessage::ProgramBuildMessage(
		const ProgramBuildMessage& rhs) :
	_programBuildId(rhs._programBuildId), _deviceIds(rhs._deviceIds),
	_buildStatus(rhs._buildStatus) {
	initSerializable();
}

ProgramBuildMessage::~ProgramBuildMessage() { }

dcl::object_id ProgramBuildMessage::programBuildId() const {
	return _programBuildId;
}

const std::vector<dcl::object_id>& ProgramBuildMessage::deviceIds() const {
    return _deviceIds;
}

void ProgramBuildMessage::getBuildStatus(
        std::vector<cl_build_status>& buildStatus) const {
    std::vector<gcf_uint32>::const_iterator src;
    std::vector<cl_build_status>::iterator dst;

    buildStatus.resize(_buildStatus.size());
    src = std::begin(_buildStatus);
    dst = std::begin(buildStatus);

    while (src != std::end(_buildStatus)) {
        *dst++ = static_cast<cl_build_status>(*src++);
    }
}

} /* namespace message */

} /* namespace dclgcf */

IMPLEMENT_SERIALIZABLE_EVENT(
        dclgcf::message::ProgramBuildMessage,
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::ProgramBuildMessage, _programBuildId);
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::ProgramBuildMessage, _deviceIds);
        ADD_ATTRIBUTE_DEFAULT(dclgcf::message::ProgramBuildMessage, _buildStatus);
        )
