/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/**
 * @file ProgramBuildMessage.h
 *
 * Messages regarding program build.
 *
 * @date 2011-12-23
 * @author Philipp Kegel
 */

#ifndef PROGRAMBUILDMESSAGE_H_
#define PROGRAMBUILDMESSAGE_H_

#include <dcl/DCLTypes.h>

#include <gcf/GCFTypedefs.h>
#include <gcf/Serializable.h>

#ifdef __APPLE__
#include <OpenCL/cl.h>
#else
#include <CL/cl.h>
#endif

#include <vector>

namespace dclgcf {

namespace message {

/**
 * @brief Notification of completion of program build.
 *
 * This message is sent from compute nodes to the host to indicate completion of program build.
 * The build status of each device is returned.
 */
class ProgramBuildMessage: public gcf::Serializable {
public:
	ProgramBuildMessage(
			dcl::object_id                      programId,
			const std::vector<dcl::object_id>&  deviceIds,
			const std::vector<cl_build_status>& buildStatus);
	ProgramBuildMessage(
	        const ProgramBuildMessage& rhs);
	virtual ~ProgramBuildMessage();

	dcl::object_id programBuildId() const;
	const std::vector<dcl::object_id>& deviceIds() const;
	void getBuildStatus(
	        std::vector<cl_build_status>& buildStatus) const;

	DECLARE_SERIALIZABLE_PUBLIC(ProgramBuildMessage, 701)

private:
	dcl::object_id _programBuildId;
	std::vector<dcl::object_id> _deviceIds;
	std::vector<gcf_uint32> _buildStatus;

	DECLARE_SERIALIZABLE_EVENT_PRIVATE(ProgramBuildMessage)
};

} /* namespace message */

} /* namespace dclgcf */

#endif /* PROGRAMBUILDMESSAGE_H_ */
