/******************************************************************************
 * This file is part of dOpenCL.
 * 
 * dOpenCL is an implementation of the OpenCL application programming
 * interface for distributed systems. See <http://dopencl.uni-muenster.de/>
 * for more information.
 * 
 * Developed by: Research Group Parallel and Distributed Systems
 *               Department of Mathematics and Computer Science
 *               University of Muenster, Germany
 *               <http://pvs.uni-muenster.de/>
 * 
 * Copyright (C) 2013  Philipp Kegel <philipp.kegel@uni-muenster.de>
 *
 * dOpenCL is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * dOpenCL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with dOpenCL. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Permission to use dOpenCL for scientific, non-commercial work is
 * granted under the terms of the dOpenCL Academic License provided
 * appropriate credit is given. See the dOpenCL Academic License for
 * more details.
 * 
 * You should have received a copy of the dOpenCL Academic License
 * along with dOpenCL. If not, see <http://dopencl.uni-muenster.de/>.
 ******************************************************************************/
/*!
 * \file    ByteBuffer.cpp
 *
 * \date    2011-04-19
 * \author  Philipp Kegel
 */

#include <dcl/Binary.h>

#include <gcf/GCFTypedefs.h>

#include <gcf/serialization/ByteBuffer.h>

#include <cassert>
#include <cstddef>
#include <cstring>

namespace gcf {

/*
 * Operator overloads for dcl::Binary
 */

template<>
ByteBuffer& ByteBuffer::operator<<(const dcl::Binary& mem) {
	/* write size */
	operator<<(static_cast<gcf_uint32>(mem.size()));

	/* write bytes */
	assert(checkSize(mem.size()));
	::memcpy(getCurrentPosition(), mem.value(), mem.size());
	offset += mem.size();

	return *this;
}

template<>
const ByteBuffer& ByteBuffer::operator>>(dcl::Binary& mem) const {
	/* read size */
	gcf_uint32 size;
	operator>>(size);

	/* read bytes */
	assert(checkSize(size));
	mem.assign(size, getCurrentPosition());
	skip(size);
	return *this;
}

} /* namespace gcf */
